from rply.token import BaseBox


class AbstractSyntaxTree:
    class BinaryOp(BaseBox):
        def __init__(self, left, right):
            self.left = left
            self.right = right

    class Number(BaseBox):
        def __init__(self, value):
            self.value = value

        def eval(self):
            return self.value

    class String(BaseBox):
        def __init__(self, value):
            self.value = value

        def eval(self):
            return self.value

    class Bool(BaseBox):
        def __init__(self, value):
            self.value = value

        def eval(self):
            return self.value

    class Add(BinaryOp):
        def eval(self):
            return self.left.eval() + self.right.eval()

    class Sub(BinaryOp):
        def eval(self):
            return self.left.eval() - self.right.eval()

    class Mul(BinaryOp):
        def eval(self):
            return self.left.eval() * self.right.eval()

    class Div(BinaryOp):
        def eval(self):
            return self.left.eval() / self.right.eval()

    class Output(BaseBox):
        def __init__(self, value):
            self.value = value

        def eval(self):
            print('>>', self.value.eval())

    class Input(BaseBox):
        def __init__(self, value):
            self.value = value

        def eval(self):
            return input(f'<< {self.value.eval()}')
