from typing import Optional

from Interpreter.Validator import Validator
from Interpreter.Lexer import Lexer
from Interpreter.Parser import Parser
from Interpreter.Repo import Repo
from Interpreter.Utils import Utils
from Models.AbstractSyntaxTree.AbstractSyntaxTree import AbstractSyntaxTree


class Interpreter:

    def __init__(self, repo: Repo, utils: Utils) -> None:
        self.__repo = repo
        self.__utils = utils
        self.__lexer = Lexer().get_lexer()
        parser = Parser()
        parser.parse()
        self.__parser = parser.get_parser()
        self.__validator = Validator(repo, utils)

    def interpret(self, line_str: str) -> None:
        """
        Interprets code line
        :param line_str:
        :return:
        """
        tokens = self.__lexer.lex(line_str)
        #for t in tokens:
        #    print(t)

        ast = self.__parser.parse(tokens)
        ast.eval()
