from rply import ParserGenerator
from Interpreter.AbstractSyntaxTree import AbstractSyntaxTree
from Models.CCLang.TokenDefinition import TokenDefinition


class Parser:
    def __init__(self):
        tokens = []
        for t in TokenDefinition:
            tokens.append(t.name)

        self.pg = ParserGenerator(
            # A list of all token names accepted by the parser.
            [
                TokenDefinition.ValueNumber.name,
                TokenDefinition.Plus.name,
                TokenDefinition.Minus.name,
                TokenDefinition.Asterisk.name,
                TokenDefinition.Slash.name,
                TokenDefinition.LeftParenthesis.name,
                TokenDefinition.RightParenthesis.name,
                TokenDefinition.Output.name,
                TokenDefinition.Semicolon.name,
                TokenDefinition.ValueString.name,
                TokenDefinition.BoolTrue.name,
                TokenDefinition.BoolFalse.name,
                TokenDefinition.Input.name
            ],
            precedence=[
                ('left', ['Plus', 'Minus']),
                ('left', ['Slash', 'Asterisk'])
            ]
        )

    def parse(self):
        #@self.pg.production('input : Input LeftParenthesis expression RightParenthesis Semicolon')
        #def statement(p):
        #    return AbstractSyntaxTree.Input(p[2])

        @self.pg.production('builtin : Output LeftParenthesis expression RightParenthesis Semicolon')
        @self.pg.production('builtin : Input LeftParenthesis expression RightParenthesis Semicolon')
        def builtin(p):
            if p[0].gettokentype() == 'Output':
                return AbstractSyntaxTree.Output(p[2])
            elif p[0].gettokentype() == 'Input':
                return AbstractSyntaxTree.Input(p[2])

        @self.pg.production('expression : ValueString')
        def expression(p):
            try:
                string = p[0].value
                if '\"' in p[0].value:
                    string = str(p[0].value).replace('\"', '')
                elif '\'' in p[0].value:
                    string = str(p[0].value).replace('\'', '')

                return AbstractSyntaxTree.String(string)
            except Exception as e:
                print(e)
                raise ValueError(p)

        @self.pg.production('expression : BoolTrue')
        @self.pg.production('expression : BoolFalse')
        def expression(p):
            try:
                return AbstractSyntaxTree.Bool(p[0].value)
            except Exception as e:
                print(e)
                raise ValueError(p)

        @self.pg.production('expression : ValueNumber')
        def expression(p):
            try:
                return AbstractSyntaxTree.Number(int(p[0].value))
            except Exception as e:
                print(e)
                raise ValueError(p)

        @self.pg.production('expression : LeftParenthesis expression RightParenthesis')
        def expression(p):
            return p[1]

        @self.pg.production('expression : expression Plus expression')
        @self.pg.production('expression : expression Minus expression')
        @self.pg.production('expression : expression Asterisk expression')
        @self.pg.production('expression : expression Slash expression')
        def expression(p):
            left = p[0]
            right = p[2]
            if p[1].gettokentype() == 'Plus':
                return AbstractSyntaxTree.Add(left, right)
            elif p[1].gettokentype() == 'Minus':
                return AbstractSyntaxTree.Sub(left, right)
            elif p[1].gettokentype() == 'Asterisk':
                return AbstractSyntaxTree.Mul(left, right)
            elif p[1].gettokentype() == 'Slash':
                return AbstractSyntaxTree.Div(left, right)
            else:
                raise AssertionError('Oops, this should not be possible!')

        @self.pg.error
        def error_handle(token):
            raise ValueError(token)

    def get_parser(self):
        return self.pg.build()
